# -*- coding: utf-8 -*-

"""
/***************************************************************************
 ForstProcessing
                                 A QGIS plugin
 ForstProcessing
 Generated by Plugin Builder: http://g-sherman.github.io/Qgis-Plugin-Builder/
                              -------------------
        begin                : 2018-11-13
        copyright            : (C) 2018 by CHL
        email                : christina_ludwig@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

__author__ = 'CHL'
__date__ = '2018-11-13'
__copyright__ = '(C) 2018 by CHL'

# This will get replaced with a git SHA1 when you do a git archive

__revision__ = '$Format:%H$'

import os
from qgis.PyQt.QtCore import QCoreApplication, QVariant, QSettings, QFileInfo, QFile, Qt
from qgis.PyQt.QtWidgets import QMessageBox, QFileDialog, QDialog, QTreeWidgetItem, QTreeWidgetItemIterator, QDialogButtonBox
from qgis.PyQt.QtGui import QIcon, QFont
from qgis.core import (QgsProcessing, QgsVectorLayer, QgsField, QgsSettings, QgsWkbTypes,
                       QgsCoordinateReferenceSystem, QgsPointXY, QgsPoint,
                       QgsProcessingAlgorithm, QgsProject, QgsApplication,
                       QgsProcessingParameterString, QgsFields, QgsVectorFileWriter,
                       QgsProcessingParameterFeatureSink, QgsFeature, QgsGeometry, QgsPalLayerSettings, QgsTextFormat,
                       QgsVectorLayerSimpleLabeling)
from qgis.gui import QgsEncodingFileDialog, QgsProjectionSelectionDialog
from .newshapefiletemplate_dialog import NewShapefileTemplateDialog
from .help_dialog import HelpDialog
from processing.gui.MessageBarProgress import MessageBarProgress
from processing.tools import dataobjects
from qgis.utils import iface
import urllib3
import certifi
import ssl

import sys
from ..resources import *
from .utils import *



class NewShapefileTemplate(QgsProcessingAlgorithm):
    """
    This is an example algorithm that takes a vector layer and
    creates a new identical one.

    It is meant to be used as an example of how to create your own
    algorithms and explain methods and variables used to do it. An
    algorithm like this will be available in all elements, and there
    is not need for additional work.

    All Processing algorithms should extend the QgsProcessingAlgorithm
    class.
    """

    # Constants used to refer to parameters and outputs. They will be
    # used when calling the algorithm from another algorithm, or when
    # calling from the QGIS console.

    OUTPUT = ""
    geometry_type = None
    srs = QgsCoordinateReferenceSystem("EPSG:32633")
    parent = None

    def initAlgorithm(self, config):
        """
        Here we define the inputs and output of the algorithm, along
        with some other properties.
        """

        # We add a feature sink in which to store our processed features (this
        # usually takes the form of a newly created vector layer when the
        # algorithm is run in QGIS).
        self.addParameter(
            QgsProcessingParameterFeatureSink(
                self.OUTPUT,
                self.tr('Output layer')
            )
        )

        # Create dialogs
        self.dlg = NewShapefileTemplateDialog()
        self.help_dlg = HelpDialog()

        # Variables
        self.dlg.btnAdd.setEnabled(False)
        self.dlg.btnRemove.setEnabled(False)

        # Connect events to functions
        self.enableConnections()
        self.set_default_crs()
        self.populate_type_box()
        #self.dlg.buttonBox.rejected.connect(self.reject)
        #self.dlg.buttonBox.accepted.connect(self.run)

    def run(self):
        self.processAlgorithm()

    def processAlgorithm(self, parameters, context, feedback):
        """
        Here is where the processing itself takes place.
        """

        print("Start processing")

        return {self.OUTPUT: "dest_id"}

    def name(self):
        """
        Returns the algorithm name, used for identifying the algorithm. This
        string should be fixed for the algorithm, and must not be localised.
        The name should be unique within each provider. Names should contain
        lowercase alphanumeric characters only and no spaces or other
        formatting characters.
        """
        return '1 - Neues Shapefile'

    def displayName(self):
        """
        Returns the translated algorithm name, which should be used for any
        user-visible display of the algorithm name.
        """
        return self.tr(self.name())

    def tr(self, string):
        return QCoreApplication.translate('Processing', string)

    def createInstance(self):
        return NewShapefileTemplate()

    def createCustomParametersWidget(self, parent):
        self.parent = parent
        return self.dlg

    def svgIconPath(self):
        return os.path.abspath("./icons/newshapefile.png")

    def icon(self):
        return QIcon(":/plugins/icons/newshapefile.png")

    def reject(self):
        self.dlg.close()

    def set_default_crs(self):
        self.srs.createFromOgcWmsCrs("EPSG:32633")
        self.srs.validate()
        self.dlg.ldtCRS.setText(str(self.srs.authid()) + " - " + str(self.srs.description()))

    # DIALOGS --------------------------------------------------------------------------------------------------------
    # ----------------------------------------------------------------------------------------------------------------

    def load_template_dialog(self):
        """
        Open file dailog so user can select the template
        :return: (User selected file, encoding)
        """
        template_dir = os.path.join(QgsApplication.qgisSettingsDirPath(), "QGIS_Vorlagen/1_Tabellen_AttributeVorlagen/")
        print(template_dir)
        encode = QSettings().value("/UI/encoding")
        file_dialog = QgsEncodingFileDialog(self.dlg, "Vorlage laden", template_dir, "Shapefiles (*.shp *.SHP)", encode)
        file_dialog.setFileMode(QFileDialog.ExistingFiles)
        file_dialog.setAcceptMode(QFileDialog.AcceptOpen)
        if not file_dialog.exec_() == QDialog.Accepted:
            return None, None
        files = file_dialog.selectedFiles()
        return str(files[0]), str(file_dialog.encoding())

    def save_template_dialog(self):
        """
        Open file dialog so user can select where the template should be saved
        :return: (User selected file, encoding)
        """
        template_dir = QFileInfo(
            QgsApplication.qgisUserDatabaseFilePath()).path() + "/QGIS_Vorlagen/1_Tabellen_AttributeVorlagen"
        encode = QSettings().value("/UI/encoding")
        file_dialog = QgsEncodingFileDialog(self.dlg, "Als Vorlage speichern", template_dir, "Shapefiles (*.shp *.SHP)", encode)
        file_dialog.setDefaultSuffix("shp")
        file_dialog.setFileMode(QFileDialog.AnyFile)
        file_dialog.setAcceptMode(QFileDialog.AcceptSave)
        file_dialog.setOption(QFileDialog.DontConfirmOverwrite, False)
        if not file_dialog.exec_() == QDialog.Accepted:
            return None, None
        files = file_dialog.selectedFiles()
        return str(files[0]), str(file_dialog.encoding())

    def save_file_dialog(self):
        """
        Open file dialog so user can select where the file should be saved
        :return:
        """
        dirName = QSettings().value("/UI/lastShapefileDir")
        encode = QSettings().value("/UI/encoding")
        file_dialog = QgsEncodingFileDialog(self.dlg, "Shapefile speichern unter ...", dirName, encoding=encode)
        file_dialog.setDefaultSuffix("shp")
        file_dialog.setFileMode(QFileDialog.AnyFile)
        file_dialog.setAcceptMode(QFileDialog.AcceptSave)
        file_dialog.setOption(QFileDialog.DontConfirmOverwrite, False)
        if not file_dialog.exec_() == QDialog.Accepted:
            return None, None
        files = file_dialog.selectedFiles()
        QSettings().setValue("/UI/lastShapefileDir", QFileInfo(str(files[0])).absolutePath())
        return str(files[0]), str(file_dialog.encoding())

    def crs_dialog(self):
        """
        Open dialog so user can select the CRS of the new shapefile
        :return:
        """
        crs_dialog = QgsProjectionSelectionDialog(self.dlg)
        crs_dialog.setCrs(self.srs)
        if not crs_dialog.exec_() == QDialog.Accepted:
            return None
        self.srs.createFromSrsId(crs_dialog.crs().srsid())
        self.dlg.ldtCRS.setText(str(self.srs.authid()) + " - " + str(self.srs.description()))

    # Save and load files ----------------------------------------------------------------------------------------
    # ------------------------------------------------------------------------------------------------------------

    def save_template(self):
        """
        Save template to file
        :return:
        """
        template_path, template_encoding = self.save_template_dialog()
        if template_path is None or template_encoding is None:
            return 1
        self.create_template(template_path, template_encoding, self.srs)

    def load_template(self):
        """
        Load attributes from template for new shapefile
        :return:
        """
        template_path, template_encoding = self.load_template_dialog()
        if template_path is None or template_encoding is None:
            return 1
        template = QgsVectorLayer(template_path, "template", "ogr")
        if not template:
            pass
            #self.iface.messageBar().pushMessage("Warnung", QApplication.translate("NewShapefileTemplate", u"Es wurde kein Vorlagenlayer geladen.", None), level=QgsMessageBar.WARNING, duration=10)
            #QMessageBox.warning(self.dlg, "achtung", u"Kein Layer geladen.")
        self.template_to_attribute_view(template)

        return 0

    def save_file(self):
        """
        Create the shapefile
        :return:
        """
        file_path, file_encoding = self.save_file_dialog()
        if file_path is None or file_encoding is None:
            return 1
        file_name = self.get_file_name(file_path)
        self.create_file(file_path, file_encoding, self.srs)
        # Load new file to canvas
        self.outlayer = QgsVectorLayer(file_path, str(file_name), "ogr")
        QgsProject.instance().addMapLayer(self.outlayer)
        QCoreApplication.processEvents()

    # Create files --------------------------------------------------------------------------------------------------
    # ---------------------------------------------------------------------------------------------------------------

    def create_template(self, template_path, encoding, crs):
        """
        Save the template to file
        :param outpath:
        :param encoding:
        :param crs:
        :return:
        """
        typ = self.selected_geometry_type()
        fields = QgsFields()
        it = QTreeWidgetItemIterator(self.dlg.AttributeView, QTreeWidgetItemIterator.All)
        while it.value():
            item = it.value()
            # Field type
            if item.text(1) == "Integer":
                Type = QVariant.Int
            elif item.text(1) == "Real":
                Type = QVariant.Double
            elif item.text(1) == "String":
                Type = QVariant.String
            else:
                Type = QVariant.Invalid
            # Field type name
            TypeName = item.data(1, Qt.UserRole)
            # Field length
            length = int(item.text(2))
            # Field precision
            if item.text(3) != "":
                precision = int(item.text(3))
            else:
                precision = 0

            fields.append(QgsField(item.text(0), Type, TypeName, length, precision))
            it += 1

        check = QFile(template_path)
        if check.exists():
            if not QgsVectorFileWriter.deleteShapeFile(template_path):
                return
        writer = QgsVectorFileWriter(template_path, encoding, fields, typ, crs, "ESRI Shapefile")
        del writer

    def create_file(self, file_path, encoding, crs):
        """
        Create the new file
        :param file_path:
        :param encoding:
        :param crs:
        :return:
        """
        fields = QgsFields()
        it = QTreeWidgetItemIterator(self.dlg.AttributeView, QTreeWidgetItemIterator.All)
        while it.value():
            item = it.value()
            field_type = QVariant.nameToType(item.text(1))
            # Field type name
            field_typename = item.data(1, 32)
            # Field length
            field_width = int(item.text(2))
            # Field precision
            if item.text(3) != "":
                field_precision = int(item.text(3))
            else:
                field_precision = 0
            fields.append(QgsField(item.text(0), field_type, field_typename, field_width, field_precision))
            it += 1

        if self.geometry_type == 1:
            if len(self.dlg.AttributeView.findItems("RECHTSWERT", Qt.MatchExactly)) == 0:
                fields.append(QgsField("RECHTSWERT", QVariant.Double, "real", 15, 4))
            if len(self.dlg.AttributeView.findItems("HOCHWERT", Qt.MatchExactly)) == 0:
                fields.append(QgsField("HOCHWERT", QVariant.Double, "real", 15, 4))
        elif self.geometry_type == 2:
            if len(self.dlg.AttributeView.findItems("METER", Qt.MatchExactly)) == 0:
                fields.append(QgsField("METER", QVariant.Double, "real", 15, 0))
        elif self.geometry_type == 3:
            if len(self.dlg.AttributeView.findItems("HEKTAR", Qt.MatchExactly)) == 0:
                fields.append(QgsField("HEKTAR", QVariant.Double, "real", 15, 4))
            if len(self.dlg.AttributeView.findItems("METER", Qt.MatchExactly)) == 0:
                fields.append(QgsField("METER", QVariant.Double, "real", 15, 0))

        check = QFile(file_path)
        if check.exists():
            if not QgsVectorFileWriter.deleteShapeFile(file_path):
                QMessageBox.warning(self.dlg, "Warnung", "Datei existiert bereits und konnte nicht gelöscht werden, da sie momentan geöffnet ist.")
                return
        writer = QgsVectorFileWriter(file_path, encoding, fields, self.geometry_type, crs, "ESRI Shapefile")
        del writer

        self.dlg.close()

    def enableConnections(self):
        """
        Connect signals to slots
        :return:
        """
        self.dlg.cbxTyp.currentIndexChanged.connect(self.correct_type)
        self.dlg.btnAdd.clicked.connect(self.add_attribute)
        self.dlg.btnRemove.clicked.connect(self.remove_attribute)
        self.dlg.ldtName.textChanged.connect(self.enable_add_button)
        self.dlg.AttributeView.itemSelectionChanged.connect(self.enable_remove_button)
        self.dlg.buttonBox.button(QDialogButtonBox.Ok).clicked.connect(self.check_geometry_type)
        self.dlg.btnLoadTemplate.clicked.connect(self.load_template)
        self.dlg.btnSaveTemplate.clicked.connect(self.save_template)
        self.dlg.btnHelp.clicked.connect(self.help_dlg.show)
        self.dlg.btnCRS.clicked.connect(self.crs_dialog)

    def add_attribute(self):
        """
        Add attribute to table view
        :return:
        """
        attr_name = self.dlg.ldtName.text()
        attr_width = self.dlg.dsbWidth.text()
        if self.dlg.dsbPrecision.isEnabled():
            precision = self.dlg.dsbPrecision.text()
        else:
            precision = self.dlg.dsbPrecision.text()

        #use userrole to avoid translated type string
        attr_type = self.dlg.cbxTyp.itemData(self.dlg.cbxTyp.currentIndex(), Qt.UserRole)
        self.dlg.AttributeView.addTopLevelItem(QTreeWidgetItem([attr_name, attr_type, attr_width, precision]))

        if self.dlg.AttributeView.topLevelItemCount() > 0:
            self.dlg.buttonBox.button(QDialogButtonBox.Ok).setEnabled(True)

        self.dlg.ldtName.clear()
        self.dlg.dsbPrecision.clear()

    def check_geometry_type(self):
        """
        Compare geometry type to attributes in table view
        :return:
        """
        typ = self.selected_geometry_type()
        if typ == 1:
            geom_name = "Punkt"
        elif typ == 2:
            geom_name = "Linie"
        else:
            geom_name = "Polygon"

        if len(self.dlg.AttributeView.findItems("Pkt_Name", Qt.MatchExactly)) > 0 and typ != QgsWkbTypes.Point:
            reply = QMessageBox.question(self.dlg, "Warnung", u"Die Attributtabelle enthält das Feld \"Pkt_Name”. Der gewählte Geometrietyp für das neue Shapefile ist jedoch %s.\n\nSoll der Layer dennoch als %s-Shapefile gespeichert werden?"%(str(geom_name),str(geom_name)), QMessageBox.Yes, QMessageBox.No)
            if reply == QMessageBox.Yes:
                self.geometry_type = self.selected_geometry_type()
                self.save_file()
            else:
                return
        elif len(self.dlg.AttributeView.findItems("Linie_Name", Qt.MatchExactly)) > 0 and typ != QgsWkbTypes.LineString:
            reply = QMessageBox.question(self.dlg, "Warnung", u"Die Attributtabelle enthält das Feld \"Linie_Name”. Der gewählte Geometrietyp für das neue Shapefile ist jedoch %s.\n\nSoll der Layer dennoch als %s-Shapefile gespeichert werden?"%(str(geom_name),str(geom_name)), QMessageBox.Yes, QMessageBox.No)
            if reply == QMessageBox.Yes:
                self.geometry_type = self.selected_geometry_type()
                self.save_file()
            else:
                return
        elif len(self.dlg.AttributeView.findItems("Kg_Name", Qt.MatchExactly)) > 0 and typ != QgsWkbTypes.Polygon:
            reply = QMessageBox.question(self.dlg, "Warnung", u"Die Attributtabelle enthält das Feld \"Gk_Name”. Der gewählte Geometrietyp für das neue Shapefile ist jedoch %s.\n\nSoll der Layer dennoch als %s-Shapefile gespeichert werden?"%(str(geom_name),str(geom_name)), QMessageBox.Yes, QMessageBox.No)
            if reply == QMessageBox.Yes:
                self.geometry_type = self.selected_geometry_type()
                self.save_file()
            else:
                return
        else:
            self.geometry_type = self.selected_geometry_type()
            self.save_file()

    def correct_type(self, index):
        """
        Check for valid input in type box
        :param index:
        :return:
        """
        if index == 0: # Text
            if (self.dlg.dsbWidth.value() < 1) or (self.dlg.dsbWidth.value() > 255):
                self.dlg.dsbWidth.setValue(80)
            self.dlg.dsbPrecision.setEnabled(False)
        elif index == 1: #whole number
            if (self.dlg.dsbWidth.value() < 1) or (self.dlg.dsbWidth.value() > 10):
                self.dlg.dsbWidth.setValue(10)
            self.dlg.dsbPrecision.setEnabled(False)
        elif index == 2: #decimal number
            if (self.dlg.dsbWidth.value() < 1) or (self.dlg.dsbWidth.value() > 20):
                self.dlg.dsbWidth.setValue(20)
            self.dlg.dsbPrecision.setEnabled(True)
            self.dlg.dsbPrecision.setValue(4)
        else:
            pass

    def enable_remove_button(self):
        """
        Enable or disable "remove attribute" button
        :return:
        """
        if len(self.dlg.AttributeView.selectedItems()) == 0:
            self.dlg.btnRemove.setDisabled(True)
        else:
            self.dlg.btnRemove.setDisabled(False)

    def enable_add_button(self):
        """
        Enable or disable "add attribute" button
        """
        name = self.dlg.ldtName.text()
        if (name == "") or (len(self.dlg.AttributeView.findItems(name, Qt.MatchExactly)) > 0):
            self.dlg.btnAdd.setDisabled(True)
        else:
            self.dlg.btnAdd.setDisabled(False)

    def get_file_name(self, file_path):
        """
        Extract file name from file path
        :param file_path:
        :return:
        """
        if "\\" in file_path:
            digit = -((len(file_path) - file_path.rfind("\\")) - 1)
            file_name = file_path[digit:]
        else:
            digit = -((len(file_path) - file_path.rfind("/")) - 1)
            file_name = file_path[digit:]
        if file_name[-4:] == ".shp":
            digit = len(file_name) - 4
            file_name = file_name[:digit]
        return file_name

    def populate_type_box(self):
        """
        Fill "typ" box with selection options
        :return:
        """
        self.dlg.cbxTyp.clear()
        self.dlg.cbxTyp.addItem("Text", QVariant(QVariant.String).typeName())
        self.dlg.cbxTyp.addItem("Ganzzahl", QVariant(QVariant.Int).typeName())
        self.dlg.cbxTyp.addItem("Dezimalzahl", QVariant(QVariant.Double).typeName())
        self.dlg.AttributeView.clear()
        self.dlg.AttributeView.addTopLevelItem(QTreeWidgetItem(["ID", QVariant(QVariant.Int).typeName(), "10", ""]))
        self.dlg.rdbPoint.setChecked(True)

    def remove_attribute(self):
        """
        Removes an attribute from the table view
        :return:
        """
        item = self.dlg.AttributeView.currentItem()
        index = self.dlg.AttributeView.indexOfTopLevelItem(item)
        self.dlg.AttributeView.takeTopLevelItem(index)
        if self.dlg.AttributeView.topLevelItemCount() == 0:
            self.dlg.buttonBox.button(QDialogButtonBox.Ok).setEnabled(False)

    def selected_geometry_type(self):
        """
        Gets the selected geometry type for the new shapefile from the dialog
        :return:
        """
        if self.dlg.rdbPoint.isChecked():
            return QgsWkbTypes.Point
        elif self.dlg.rdbLine.isChecked():
            return QgsWkbTypes.LineString
        elif self.dlg.rdbPolygon.isChecked():
            return QgsWkbTypes.Polygon
        else:
            return QgsWkbTypes.Unknown

    def template_to_attribute_view(self, template):
        """
        Load attributes from template to attribute view of new shapefile
        :param template: QgsVectorLayer containing attributes
        :return:
        """
        attributes = template.fields()
        self.dlg.AttributeView.clear()
        for field in attributes:
            field_name = field.name()
            field_type = QVariant.typeToName(field.type())
            field_width = str(field.length())
            field_precision = str(field.precision())
            if field_precision == "0":
                field_precision = ""

            self.dlg.AttributeView.addTopLevelItem(QTreeWidgetItem([field_name, field_type, field_width, field_precision]))

        if self.dlg.AttributeView.itemAt(0, 0) is not None:
            self.dlg.buttonBox.button(QDialogButtonBox.Ok).setEnabled(True)

        return 0
