"""
***************************************************************************
Name			 	 : Geocoding
Description          : Geocoding and reverse Geocoding using Google
Date                 : 28/May/09
copyright            : (C) 2009-2013 by ItOpen
email                : info@itopen.it
 ***************************************************************************/

 /***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

from qgis.core import QgsVectorLayer, QgsPointXY, QgsGeometry, QgsCoordinateReferenceSystem, QgsCoordinateTransform, QgsProject, QgsRasterLayer
from shapely.geometry import Polygon, Point
import numpy as np
from qgis.gui import *

class ClickTool(QgsMapTool):
    def __init__(self,iface, callback):
        QgsMapTool.__init__(self,iface.mapCanvas())
        self.iface = iface
        self.callback = callback
        self.canvas = iface.mapCanvas()
        return None


    def canvasReleaseEvent(self,e):
        point = self.canvas.getCoordinateTransform().toMapPoint(e.pos().x(),e.pos().y())
        self.callback(point)
        return None


def pointToWGS84(point, crs):
    """
    crs is the renderer crs
    """
    target =QgsCoordinateReferenceSystem("EPSG:4326")
    source = crs #QgsCoordinateReferenceSystem()
    #f.createFromProj4(proj4string)
    transformer = QgsCoordinateTransform(source, target, QgsProject.instance())
    pt = transformer.transform(point)
    return pt

def pointTo3857(point, crs):
    """
    crs is the renderer crs
    """
    target = QgsCoordinateReferenceSystem("EPSG:3857")
    source = crs #QgsCoordinateReferenceSystem()
    #f.createFromProj4(proj4string)
    transformer = QgsCoordinateTransform(source, target, QgsProject.instance())
    pt = transformer.transform(point)
    return pt

def pointFromWGS84(point, crs):
    source = QgsCoordinateReferenceSystem("EPSG:4326")
    target = crs
    #t.createFromProj4(proj4string)
    print(type(source))
    print(type(target))
    transformer = QgsCoordinateTransform(source, target, QgsProject.instance())
    pt = transformer.transform(point)
    return pt

def layer_names(geom_type):
    layermap = QgsProject().instance().mapLayers()
    layerlist = []
    if geom_type == "all":
        layerlist = [name for name, layer in layermap.iteritems()]
        print(layerlist)
    else:
        for name, layer in layermap.iteritems():
            if isinstance(layer, QgsVectorLayer):
                if layer.geometryType() in geom_type:
                    layerlist.append(layer.name())
            elif isinstance(layer, QgsRasterLayer):
                if "Raster" in geom_type:
                    layerlist.append(layer.name())
    return sorted( layerlist, cmp=locale.strcoll)

def random_points_within(poly, num_points):
    bbox = poly.boundingBox()
    min_x, min_y, max_x, max_y = bbox.xMinimum(), bbox.yMinimum(), bbox.xMaximum(), bbox.yMaximum()
    points = []
    while len(points) < num_points:
        random_point = QgsGeometry.fromPointXY(QgsPointXY(np.random.uniform(min_x, max_x), np.random.uniform(min_y, max_y)))
        if poly.contains(random_point):
            points.append(random_point)
    return points
