# -*- coding: utf-8 -*-

"""
/***************************************************************************
 ForstProcessing
                                 A QGIS plugin
 ForstProcessing
 Generated by Plugin Builder: http://g-sherman.github.io/Qgis-Plugin-Builder/
                              -------------------
        begin                : 2018-11-13
        copyright            : (C) 2018 by CHL
        email                : christina_ludwig@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

__author__ = 'CHL'
__date__ = '2018-11-13'
__copyright__ = '(C) 2018 by CHL'

# This will get replaced with a git SHA1 when you do a git archive

__revision__ = '$Format:%H$'

import os
from PyQt5.QtCore import QCoreApplication, QVariant
from PyQt5.QtWidgets import QMessageBox
from qgis.PyQt.QtGui import QIcon, QFont
from qgis.core import (QgsProcessing, QgsVectorLayer, QgsField, QgsSettings, QgsProject, QgsWkbTypes,
                       QgsCoordinateReferenceSystem, QgsPointXY, QgsPoint,
                       QgsProcessingAlgorithm,
                       QgsProcessingParameterString,
                       QgsProcessingParameterFeatureSink, QgsFeature, QgsGeometry, QgsPalLayerSettings, QgsTextFormat,
                       QgsVectorLayerSimpleLabeling)
from .addtraversepoints_dialog import AddTraversePointsDialog
from .help_dialog import HelpDialog
from processing.gui.MessageBarProgress import MessageBarProgress
from processing.tools import dataobjects
from qgis.utils import iface
import math
from ..resources import *
from .utils import *


class AddTraversePoints(QgsProcessingAlgorithm):
    """
    This is an example algorithm that takes a vector layer and
    creates a new identical one.

    It is meant to be used as an example of how to create your own
    algorithms and explain methods and variables used to do it. An
    algorithm like this will be available in all elements, and there
    is not need for additional work.

    All Processing algorithms should extend the QgsProcessingAlgorithm
    class.
    """

    # Constants used to refer to parameters and outputs. They will be
    # used when calling the algorithm from another algorithm, or when
    # calling from the QGIS console.

    ADDRESS = ""
    OUTPUT = ""
    parent = None
    points = []
    canvas = iface.mapCanvas()

    def initAlgorithm(self, config):
        """
        Here we define the inputs and output of the algorithm, along
        with some other properties.
        """
        self.parent = None

        # We add the input vector features source. It can have any kind of
        # geometry.
        self.addParameter(
            QgsProcessingParameterString(
                self.ADDRESS,
                'Adresse'
            )
        )

        # We add a feature sink in which to store our processed features (this
        # usually takes the form of a newly created vector layer when the
        # algorithm is run in QGIS).
        self.addParameter(
            QgsProcessingParameterFeatureSink(
                self.OUTPUT,
                self.tr('Output layer')
            )
        )

        self.dlg = AddTraversePointsDialog()
        self.help_dlg = HelpDialog()

        self.dlg.btnHelp.clicked.connect(self.help_dlg.show)
        self.dlg.buttonBox.rejected.connect(self.reject)
        self.dlg.btnCreatePoint.clicked.connect(self.run)
        QgsProject().instance().layerRemoved.connect(self.add_layers_to_combobox)
        QgsProject().instance().layerWasAdded.connect(self.add_layers_to_combobox)
        self.add_layers_to_combobox()

    def run(self):
        if self.check_input() == 0:
            self.processAlgorithm()


    def check_input(self):
        """
        Check input parameters
        :return:
        """
        self.layer = QgsProject().instance().mapLayersByName(self.dlg.cbx_layers.currentText())[0]
        if len(self.layer.selectedFeatures()) < 1:
            QMessageBox.warning(self.dlg, "Warnung",
                                u"Bitte wählen Sie mit Hilfe dem Auswahlwerkzeuges in der "
                                u"Werkzeugleiste ein Feature als ersten Referenzpunkt aus.")
            return 1
        elif len(self.layer.selectedFeatures()) > 1:
            QMessageBox.warning(self.dlg, "Warnung",
                                u"Es sind zu viele Features selektiert.\n\nBitte selektieren Sie nur ein Feature.")
            return 1
        elif not self.layer.isEditable():
            QMessageBox.warning(self.dlg, "Warnung", "Der Layer ist nicht im Editiermodus.")
            return 1
        elif self.dlg.dsbDistance.value() == 0:
            QMessageBox.warning(self.dlg, "Warnung",
                                u"Der Wert der Entfernung ist 0.\n\nBitte geben Sie einen gültigen Wert ein.")
            return 1
        else:
            return 0

    def processAlgorithm(self):
        """
        Here is where the processing itself takes place.
        """
        result = self.create_new_point()
        return {self.OUTPUT: "dest_id"}

    def point_coordinates(self, feature):
        if not feature.geometry().isMultipart():
            geom = feature.geometry().asPoint()
        else:
            geom = feature.geometry()
            geom.convertToSingleType()
            geom = geom.asPoint()
        return (geom.x(), geom.y())

    def next_point_coordinates(self, current_point, distance, azimuth):
        x, y = self.point_coordinates(current_point)
        azimuth_rad = math.radians(azimuth)
        new_x = x + distance * math.sin(azimuth_rad)
        new_y = y + distance * math.cos(azimuth_rad)
        return (new_x, new_y)

    def create_new_point(self):
        """
        Creates the next point based on the current point, distance and azimuth
        :return:
        """

        current_point = self.get_current_point()
        if current_point is None:
            return 1
        distance, azimuth = self.get_azimuth_distance_parameters()
        new_x, new_y = self.next_point_coordinates(current_point, distance, azimuth)

        # Create new feature
        new_id = self.layer.featureCount() + 1
        new_feature = QgsFeature(self.layer.fields())
        new_point = QgsPointXY(new_x, new_y)
        new_feature.setGeometry(QgsGeometry.fromPointXY(new_point))
        new_feature.setAttribute(0, new_id)

        # Add new feature to layer and select it
        self.layer.addFeature(new_feature)
        self.layer.removeSelection()
        self.layer.select(new_feature.id())

        # Pan to new point if it is not inside the canvas anymore
        self.canvas.refresh()
        extent = self.canvas.extent()
        pgeom = new_feature.geometry()
        if not pgeom.intersects(extent):
            self.canvas.panToSelected()
        return 0

    def get_current_point(self):
        """Get currently selected feature from point layer"""
        selectedFeatures = list(self.layer.getSelectedFeatures())
        if len(selectedFeatures) != 0:
            return selectedFeatures[0]
        else:
            return None

    def get_azimuth_distance_parameters(self):
        """
        Get azimuth and distance parameters from dialog
        :return: 
        """
        # Get distance value
        distance = self.dlg.dsbDistance.value()
        # Get Azimuth
        if self.dlg.rdbDecDeg.isChecked() and self.dlg.AltNeu.currentIndex() == 0: # Altgrad Dezimal
            azimuth = self.dlg.dsbAzimuth.value()
        elif self.dlg.rdbNeuDecDeg.isChecked() and self.dlg.AltNeu.currentIndex() == 1: # Neugrad Dezimal
            NeuAz = self.dlg.dsbNeuAzimuth.value()
            azimuth = NeuAz * 0.90
        elif self.dlg.rdbDMS.isChecked() and self.dlg.AltNeu.currentIndex() == 0: # Altgrad D/M/S
            deg = self.dlg.dsbDeg.value() 
            min = self.dlg.dsbMin.value()
            sec = self.dlg.dsbSec.value()
            azimuth = (((sec/60.00)+min)/60.00)+deg
        else: # Neugrad D / M / S
            Ndeg = self.dlg.dsbNeuDeg.value() 
            Nmin = self.dlg.dsbNeuMin.value()
            Nsec = self.dlg.dsbNeuSec.value()
            decdeg = Ndeg + Nmin/100.00 + Nsec/10000.00
            azimuth = decdeg * 0.90
        return distance, azimuth
            
            
    def name(self):
        """
        Returns the algorithm name, used for identifying the algorithm. This
        string should be fixed for the algorithm, and must not be localised.
        The name should be unique within each provider. Names should contain
        lowercase alphanumeric characters only and no spaces or other
        formatting characters.
        """
        return '4 - Polares Anhängen'

    def displayName(self):
        """
        Returns the translated algorithm name, which should be used for any
        user-visible display of the algorithm name.
        """
        return self.tr(self.name())

    def tr(self, string):
        return QCoreApplication.translate('Processing', string)

    def createInstance(self):
        return AddTraversePoints()

    def createCustomParametersWidget(self, parent):
        self.parent = parent
        return self.dlg

    def svgIconPath(self):
        return os.path.abspath("./icons/add_traverse_points.png")

    def icon(self):
        return QIcon(":/plugins/icons/add_traverse_points.png")

    def reject(self):
        self.dlg.close()

    def add_layers_to_combobox(self):
        """
        Add all polygon vector layers to the change box
        :return:
        """
        project = QgsProject().instance()
        layers = []
        for lay in project.mapLayers():
            if isinstance(project.mapLayer(lay), QgsVectorLayer):
                if project.mapLayer(lay).geometryType() == 0:
                    layers.append(project.mapLayer(lay).name())
        self.dlg.cbx_layers.clear()
        self.dlg.cbx_layers.addItems(layers)

